<?php
/**
 * @file
 * Drupal Ipsum pages.
 */

/**
 * Form callback.
 * Returns the Drupal Ipsum text generation form.
 *
 * @see drupal_ipsum_menu()
 */
function drupal_ipsum_generate_form($form, &$form_state, $vocabulary = NULL, $length = NULL, $type = NULL, $start = NULL, $html = NULL) {
  // Override defaults if we've already submitted the form.
  if (isset($form_state['storage'])) {
    extract($form_state['storage'], EXTR_OVERWRITE);

    $form['results'] = array(
      '#theme' => 'drupal_ipsum_results',
      '#text' => $drupal_ipsum_results,
      '#weight' => -9,
    );

    $submit_title = t('Generate more!');
  }
  else {
    $submit_title = t('Generate!');
  }

  $form += drupal_ipsum_get_form($vocabulary, $length, $type, $start, $html);

  $form['#attached']['css'] = array(drupal_get_path('module', 'drupal_ipsum') . '/css/drupal_ipsum.css');
  $form['#attached']['js'] = array(drupal_get_path('module', 'drupal_ipsum') . '/js/drupal_ipsum.js');

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $submit_title,
  );

  return $form;
}

/**
 * Validation callback.
 *
 * @see drupal_ipsum_generate_form()
 */
function drupal_ipsum_generate_form_validate($form, &$form_state) {
  // Alias for convenience.
  $values = & $form_state['values'];

  // Load available vocabularies.
  $drupal_ipsum = Drupal_Ipsum::getInstance();
  $vocabularies = $drupal_ipsum->getVocabularies();

  // Validate vocabulary.
  if (!in_array($values['drupal_ipsum_vocabulary'], array_keys($vocabularies))) {
    form_set_error('drupal_ipsum_vocabulary', t('Invalid vocabulary!'));
  }

  // Validate type.
  if (!is_numeric($values['drupal_ipsum_type']) || !($values['drupal_ipsum_type'] >= 0) || !($values['drupal_ipsum_type'] <= 2)) {
    form_set_error('drupal_ipsum_type', t('Invalid text generation type!'));
  }

  // Validate start.
  if (!is_numeric($values['drupal_ipsum_start']) || !($values['drupal_ipsum_start'] >= 0) || !($values['drupal_ipsum_start'] <= 2)) {
    form_set_error('drupal_ipsum_start', t('Invalid text generation start!'));
  }

  // Validate length.
  if (!is_numeric($values['drupal_ipsum_length'])) {
    form_set_error('drupal_ipsum_length', t('Quantity must be numeric!'));
  }
}

/**
 * Submit callback.
 *
 * @see drupal_ipsum_generate_form()
 */
function drupal_ipsum_generate_form_submit(&$form, &$form_state) {
  // Alias for convenience.
  $values = & $form_state['values'];

  // Generate text.
  $text = drupal_ipsum_generate(
    $values['drupal_ipsum_vocabulary'],
    $values['drupal_ipsum_length'],
    $values['drupal_ipsum_type'],
    $values['drupal_ipsum_start'],
    $values['drupal_ipsum_html']
  );

  // Prepare variables for feedback message.
  $vocabulary_key = $values['drupal_ipsum_vocabulary'];
  $drupal_ipsum = Drupal_Ipsum::getInstance();
  $vocabularies = $drupal_ipsum->getVocabularies();
  $vocabulary_name = $vocabularies[$vocabulary_key];

  // Either this or have 3 different format_plural() calls for each type.
  // I kinda like this better. If it doesn't translate properly or something
  // please let me know! :)
  switch ($values['drupal_ipsum_type']) {
    case DRUPAL_IPSUM_TYPE_WORD:
      $type_singular = 'word';
      $type_plural = 'words';
      break;
    case DRUPAL_IPSUM_TYPE_SENTENCE:
      $type_singular = 'sentence';
      $type_plural = 'sentences';
      break;
    case DRUPAL_IPSUM_TYPE_PARAGRAPH:
      $type_singular = 'paragraph';
      $type_plural = 'paragraphs';
      break;
  }

  // Format and display confirmation message.
  $summary = format_plural($values['drupal_ipsum_length'], 'Generated 1 %type_singular of Drupal Ipsum filler text using the %vocabulary vocabulary!', 'Generated @count %type_plural of Drupal Ipsum filler text using the %vocabulary vocabulary!', array('%type_singular' => $type_singular, '%type_plural' => $type_plural, '%vocabulary' => $vocabulary_name));
  drupal_set_message($summary, 'status', FALSE);

  // Persist values and rebuild form.
  $form_state['storage']['drupal_ipsum_results'] = $text;
  $form_state['storage']['vocabulary'] = $values['drupal_ipsum_vocabulary'];
  $form_state['storage']['length'] = $values['drupal_ipsum_length'];
  $form_state['storage']['type'] = $values['drupal_ipsum_type'];
  $form_state['storage']['start'] = $values['drupal_ipsum_start'];
  $form_state['storage']['html'] = $values['drupal_ipsum_html'];
  $form_state['rebuild'] = TRUE;
}
